/*++

Copyright (c) Sydus.

Module:

    Selector.h

Authors:

    Sharvil Nanavati (sharvil) 2006-07-29

--*/

#pragma once

#include "base.h"
#include "ISelectorImplementation.h"

class Selector
{
	public:
		Selector();
		~Selector();

		void Alert();                                        // Safe to call from another thread.

		int32 Select();
		int32 Select(uint32 timeoutMillis);

		const SelectedSet & AcceptSet() const;
		const SelectedSet & ReadSet() const;                // Returns a set that's been selected for read
		const SelectedSet & WriteSet() const;               // Returns a set that's been selected for write

		bool IsAlerted() const;                             // Whether the last Select was Alerted

		enum
		{
			OP_ACCEPT = 1,
			OP_READ = 2,
			OP_WRITE = 4
		};

	private:
		friend class ISelectable;
		bool Register(int32 fd, ISelectable & selectable);
		void Unregister(int32 fd);

		ISelectorImplementation * iPimpl;
};
