/*++

Copyright (c) Sydus.

Module:

	Nucleus_String.h

Authors:

	Sharvil Nanavati (sharvil) 2006-09-19

--*/

#pragma once

#include "base.h"

#include <vector>
#include <set>

typedef std::vector <std::string> StringSequence;
typedef std::set <std::string> StringSet;

namespace String
{
	enum ECaseSensitivity
	{
		KCaseSensitive,
		KCaseInsensitive
	};

	int32 Compare(const_cstring left, const_cstring right, ECaseSensitivity sensitivity = KCaseSensitive);
	int32 Compare(const std::string & left, const std::string & right, ECaseSensitivity sensitivity = KCaseSensitive);

	std::string & ToUpper(std::string & str);
	std::string & ToLower(std::string & str);

	std::string ToUpperCopy(const std::string & str);
	std::string ToLowerCopy(const std::string & str);

	std::string & Trim(std::string & str);
	std::string & LTrim(std::string & str);
	std::string & RTrim(std::string & str);

	std::string TrimCopy(const std::string & str);
	std::string LTrimCopy(const std::string & str);
	std::string RTrimCopy(const std::string & str);

	StringSequence Split(const std::string & source, std::string::value_type split);
	std::string Join(const StringSequence & source, std::string::value_type join);

	struct CaseInsensitiveComparator
	{
		bool operator() (const std::string & left, const std::string & right) const { return String::Compare(left, right, KCaseInsensitive) < 0; }
	};

	struct CaseSensitiveComparator
	{
		bool operator() (const std::string & left, const std::string & right) const { return String::Compare(left, right, KCaseSensitive) < 0; }
	};
}

typedef std::set <std::string, String::CaseInsensitiveComparator> StringISet;
